function [indexChans] = goodChannelIndices(dataTable,hemisphere,completeChannelSet,printDebug)
%goodChannelIndices Considers bad channels and hemisphere/cross-set options
%   This function also creates correct channel ordering for patients with
%   specified "contact" IDs rather than channels
%   Options:
%   - hemisphere, string: 'dominant'/'other','R','L','both'; required
%   - completeChannelSet: if true, use consistent set across all rows from
%   the same subject.
%   - printDebug; set to false to prevent debug output
%   
%
%   Examples of use afterwards:
%   Add to table with:
%       dataTable = addvars(dataTable,indexChans);
%
%   Select only relevant ECoGchannels (and order correctly):
%       dataTable.ECoGchannels{iRow} = dataTable.ECoGchannels{iRow}(indexChans{iRow});
%
%   Select only relevant raw data channels (and order correctly), requires an additional function:
%       test.rawData{iRow} = structfun(@(x) selectRaw(x,indexChans{iRow}),test.rawData{iRow},'UniformOutput',false);
%       function [x] = selectRaw(x,indices)
%           x.data = x.data(:,indices);
%       end

if nargin<4
    printDebug = true;
end

indexChans = cell(size(dataTable,1),1);

for iRow = 1:size(dataTable,1)
    %% Discard bad channels from own data record
    logicChans = true(size(dataTable.ECoGchannels{iRow}));
    logicChans(dataTable.badChannels{iRow}) = false;
    %~ismember([dataTable.ECoGchannels{iRow}.chanNum],dataTable.badChannels{iRow});
    chansFromRow = sum(logicChans);
    
    %% Keep channels according to hemisphere options
    switch hemisphere
        case 'dominant'
            logicChans = logicChans & ~ecogutils.getContralateralChannels(dataTable.ECoGchannels(iRow));
        case 'other'
            logicChans = logicChans & ecogutils.getContralateralChannels(dataTable.ECoGchannels(iRow));
        case {'R','right'}
            logicChans = logicChans & dataTable.ECoGchannels{iRow}.MNIX > 0;
        case {'L','left'}
            logicChans = logicChans & dataTable.ECoGchannels{iRow}.MNIX < 0;
        case 'keepnan'
            logicChans = logicChans & (~ecogutils.getContralateralChannels(dataTable.ECoGchannels(iRow)) | isnan([dataTable.ECoGchannels{iRow}.MNIX]));
        case 'both'
            %do nothing
        otherwise
            error(['Unknown hemisphere option: ' options.hemisphere]);
    end
    chansFromHemisphere = sum(logicChans);
    
    
    %% Discard bad channels from other rows from same patient
    if completeChannelSet
        for jRow = 1:size(dataTable,1)
            if jRow~=iRow && strcmp(dataTable.patientID{iRow},dataTable.patientID{jRow})
                
                if isfield(dataTable.ECoGchannels{iRow},'contNum')
                    ourChannels = [dataTable.ECoGchannels{iRow}.contNum];
                else
                    ourChannels = [dataTable.ECoGchannels{iRow}.chanNum];
                end
                
                if isfield(dataTable.ECoGchannels{jRow},'contNum')
                    theirChannels = [dataTable.ECoGchannels{jRow}.contNum];
                else
                    theirChannels = [dataTable.ECoGchannels{jRow}.chanNum];
                end
                theirLogic = true(size(dataTable.ECoGchannels{jRow}));
                theirLogic(dataTable.badChannels{jRow}) = false;
                theirChannels = theirChannels(theirLogic);
                
                logicChans = logicChans & ismember(ourChannels,theirChannels);
                
            end
        end
    end
    chansFromOthers = sum(logicChans);
    
    %%
    if printDebug
        disp(['Channels kept by step: ' num2str(chansFromRow) ' ' num2str(chansFromHemisphere) ' ' num2str(chansFromOthers)]);
    end
    
    if isfield(dataTable.ECoGchannels{iRow},'contNum')
        [~,contactOrder] = sort([dataTable.ECoGchannels{iRow}(logicChans).contNum]);
    else
        [~,contactOrder] = sort([dataTable.ECoGchannels{iRow}(logicChans).chanNum]);
    end
    indexChans{iRow} = find(logicChans);
    indexChans{iRow} = indexChans{iRow}(contactOrder);
end


end

