function [remapped] = mapEmbedding(embedding,sourceEigVecs,destEigVecs)
% mapEmbedding Remaps the matrix of row vectors embedding from the space 
% spanned by sourceEigVecs to the space spanned by destEigVecs.
%
% IMPORTANT: 
% embedding = matrix whose rows are the points in embedding space 
% represented by row vectors, and whose columns are the dimensions of that 
% space.
% sourceEigVecs and destEigVecs = matrices whose columns are the right
% eigenvectors of the source and dest spaces.
%
% For a single vector v the remapping from source space to dest space is as
% shown in section 3.4 from Coifman and Hirn 2014:
% 
% v_remapped[i] = sum_over_j{v[j]*inner_prod((eigVec_dest_i),(eigVec_source_j)).
% 
% We have a matrix of (row) vectors we want to remap.
% In our code, the tempDot loop just does all those inner products up front. Then the loop
%  
%         for iCol = 1:nDim
%             remapped(iPt,iCol) =...
%                 embedding(iPt,:)*tempDot(iCol,:)';
%         end
% 
% does the sum_over_j operation for the iPt^th vector.
% 
% To verify that this is set up correctly, consider a 2-D space. Then 
% tempDot is as follows:
% 
% tempDot = [dot(eig_dest_1,eig_source_1), dot(eig_dest_1,eig_source_2); 
%            dot(eig_dest_2,eig_source_1), dot(eig_dest_2,eig_source_2)];
% 
% tempDot' = [dot(eig_dest_1,eig_source_1), dot(eig_dest_2,eig_source_1); 
%             dot(eig_dest_1,eig_source_2), dot(eig_dest_2,eig_source_2)];
% 
% and the remapped loop above multiplies the source embedding row vectors 
% by tempDot'. Then the two remapped embedding row vectors are
% 
% remapped(1,1) = embedding(1,1)*dot(eig_dest_1,eig_source_1)+embedding(1,2)*dot(eig_dest_1,eig_source_2);
% remapped(1,2) = embedding(1,1)*dot(eig_dest_2,eig_source_1)+embedding(1,2)*dot(eig_dest_2,eig_source_2);
% 
% remapped(2,1) = embedding(2,1)*dot(eig_dest_1,eig_source_1)+embedding(2,2)*dot(eig_dest_1,eig_source_2);
% remapped(2,2) = embedding(2,1)*dot(eig_dest_2,eig_source_1)+embedding(2,2)*dot(eig_dest_2,eig_source_2);
% 
% Original code whose notation was updated 23 Aug 2022 by MIB:
%     nDim = size(embedding,1);
%     tempDot = zeros(nDim);
%     for iCol = 1:nDim
%         for j = 1:nDim
%             tempDot(iCol,j) = dot(destEigVecs(:,iCol),sourceEigVecs(:,j));
%         end
%     end
%     remapped = zeros(nDim);
%     for iPt = 1:nDim
%         for iCol = 1:nDim
%             remapped(iPt,iCol) =...
%                 embedding(iPt,:)*tempDot(iCol,:)';
%         end
%     end
%
    [nPt,nDim] = size(embedding);
    tempDot = zeros(nDim);
    for iEigDest = 1:nDim
        for jEigSource = 1:nDim
            tempDot(iEigDest,jEigSource) = dot(destEigVecs(:,iEigDest),sourceEigVecs(:,jEigSource));
        end
    end
    remapped = zeros(nPt,nDim);
    for iPt = 1:nPt
        for jDim = 1:nDim
            remapped(iPt,jDim) =...
                embedding(iPt,:)*tempDot(jDim,:)';
        end
    end
end

