classdef AdjMatrix < handle
    %AdjMatrix
    %   Class to handle adjacency matrices with labels

    properties (SetAccess = private)
        Adj double
        MetaData struct
        Emb Embedding
    end
    properties (Dependent,SetAccess = private)
        Contacts double
        ROI cell
    end

    methods
        newObj = avgByROI(obj,ROIs);
        sortByROI(obj,ROI_table,majorROI,minorROI,suppressWarning);

        function adj = double(obj)
            % When cast as double, return just the adjacency matrix
            adj = obj.Adj;
        end

        function roi = get.ROI(obj)
            if isfield(obj.MetaData,'ROI')
                roi = {obj.MetaData.ROI};
            elseif isfield(obj.MetaData,'oldROI')
                roi = {obj.MetaData.oldROI};
            else
                error('MetaData does not contain ROI information');
            end
        end

        function set.ROI(obj,roi)
            if isfield(obj.MetaData,'ROI')
                [obj.MetaData.ROI] = roi{:};
            elseif isfield(obj.MetaData,'oldROI')
                [obj.MetaData.oldROI] = roi{:};
            else
                error('MetaData does not contain ROI information');
            end
        end

        function cont = get.Contacts(obj)
            if isfield(obj.MetaData,'contNum')
                cont = [obj.MetaData.contNum];
            else
                error('MetaData does not contain contact information - is this a contact x contact matrix?');
            end
        end

        function obj = index(obj,ia)
            if numel(obj)>1
                error('Method index only applies to singular object');
            end
            if length(ia) ~= length(unique(ia))
                error('Index values are not unique');
            end
            obj.Adj = obj.Adj(ia,ia,:);
            obj.MetaData = obj.MetaData(ia);
            % Embedding is not valid after sorting
            obj.Emb = Embedding.empty;
        end

        function [obj,obj2] = matchContacts(obj,obj2)
            % Index adjacency matrix to another, such that they share the
            % same contacts in the same order
            if numel(obj)>1
                error('Method matchContacts only applies to singular object');
            end
            [~,ia,ib] = intersect(obj.Contacts,obj2.Contacts,'stable');

            obj = obj.index(ia);
            obj2 = obj.index(ib);
        end

        function obj = AdjMatrix(A,metaData)
            %AdjMatrix Construct an instance of this class
            %   A is an adjacency matrix; metaData is a structure describing the rows/columns of the adjacency matrix.
            %   metaData must include ROI or oldROI, contact number
            if nargin == 0
                return;
            end
            if ~isfield(metaData,'ROI') && ~isfield(metaData,'oldROI')
                error('metaData must include ROI or oldROI');
            end
            obj.Adj = A;
            obj.MetaData = metaData;
        end


        function emb = embed(obj,varargin)
            %embed Diffusion map embedding of this adjacency matrix
            %   Detailed explanation goes here
            for ii = 1:numel(obj)
                emb = Embedding(obj(ii).Adj,varargin{:});
                obj(ii).Emb = emb;
            end
        end

        function remapROIs(obj,ROImap)
            %remapROIs Uses containers.map ROImap to map ROIs to a
            %different ROI scheme. ROIs without a key in ROImap are
            %unaffected.
            for ii = 1:numel(obj)
                oldROIs = obj(ii).ROI;
                oldROIs(isKey(ROImap,oldROIs)) = values(ROImap,oldROIs(isKey(ROImap,oldROIs)));
                obj(ii).ROI = oldROIs;
            end
        end
    end
end