classdef Embedding < handle
    %Embedding
    %   Diffusion map representation of an adjacency matrix

    properties (SetAccess = private)
        EigenValues
        EigenVectors
        SymmP
        Emb
        t
        Method
        Threshold
        Elbow
    end

    methods(Access = private)
        function resetElbow(obj)
            obj.Elbow = [];
        end
    end

    methods
        effDim = effectiveDimensionality(obj);
        distances = embDistUsingElbow(obj,maxDimension);
        [nearVsMedian,sortedDistance] = localDifferentiation(obj,maxDimension);

        function emb = double(obj)
            emb = obj.Emb;
        end

        function mappedEmb = map(obj,template)
            mappedEmb = obj.Emb;
            mappedEmb(:,2:end) = ecogutils.mapEmbedding(obj.Emb(:,2:end),obj.EigenVectors(:,2:end),template.EigenVectors(:,2:end));
        end

        function elbow = get.Elbow(obj)
            if isempty(obj.Elbow)
               kneeOut = py.kneed.KneeLocator(1:(length(obj.EigenValues)-1),abs(obj.EigenValues(2:end)),pyargs('curve','convex','direction','decreasing','S',0.01));

                if isa(kneeOut.knee,'double')
                    obj.Elbow = kneeOut.knee;
                else
                    obj.Elbow = NaN;
                end
                obj.Elbow = max(obj.Elbow,5);
            end
            elbow = obj.Elbow;
        end

        function set.EigenValues(obj,value)
            obj.EigenValues = value;
            resetElbow(obj);
        end

        function emb = Embedding(adjMat,threshold,method,t)
            %Embedding Construct an instance of this class
            %   Threshold and calculate embedding
            if nargin~=0
                if nargin<4 || isempty(t), t = 1; end
                if nargin<3 || isempty(method), method = 'raw'; end

                sz = size(adjMat);
                if sz(1)~=sz(2), error('Adjacency matrix not square'), end
                if length(sz)>3, error('Only Ch x Ch x N matrices supported'), end
                if length(sz)==2, N = 1; else, N = sz(3); end

                emb(N,1) = Embedding;
                nRow = sz(1);
                [emb.t] = deal(t);
                [emb.Threshold] = deal(threshold);

                for iEmb = 1:N
                    thisAdj = adjMat(:,:,iEmb);
                    %% Thresholding
                    % Negative to zero
                    thisAdj(thisAdj<0) = 0;


                    if ~strcmp(method,'cosinePreThresh')
                        % MST to ensure graph is connected
                        G = graph(1./thisAdj);
                        MST = full(adjacency(minspantree(G)));

                        % Keep k=threshold nearest neighbors

                        knn = ecogutils.knnThresh(thisAdj,floor(size(thisAdj,1)*threshold));
                        thisAdj(~(MST | knn)) = 0;
                    end


                    %% Cosine similarity, if selected
                    switch method
                        case {'cosine','cosinePreThresh'}
                            if strcmp(method,'cosinePreThresh')
                                emb(iEmb).Method = 'CosineSimilarityPreThresh';
                            else
                                emb(iEmb).Method = 'CosineSimilarity';
                            end
                            K = zeros(nRow);
                            for iRow = 1:nRow
                                thisRow = thisAdj(iRow,:);
                                thisNorm = norm(thisRow);
                                for jRow = 1:nRow
                                    thatRow = thisAdj(jRow,:);
                                    thatNorm = norm(thatRow);
                                    if thisNorm>0 && thatNorm>0
                                        K(iRow,jRow) = dot(thisRow,thatRow)/(thisNorm*thatNorm);
                                    end
                                end
                            end
                            %                             K = K.*(1-eye(nRow));
                        case 'taxicab'
                            K = squareform(pdist(thisAdj,'minkowski',1));
                            K = 1./(1+K);
                        case 'raw'
                            emb(iEmb).Method = 'Raw';
                            % Make K perfectly symmetric (even if K is already symmetric,
                            % this step ensures symmetry to numerical precision
                            K = (thisAdj + thisAdj')/2;
                        otherwise
                            error('Unsupported method. Supported methods are: cosine, minkowski, raw');
                    end

                    if strcmp(method,'cosinePreThresh')
                        % MST to ensure graph is connected
                        G = graph(1./K);
                        MST = full(adjacency(minspantree(G)));

                        % Keep k=threshold nearest neighbors

                        knn = ecogutils.knnThresh(K,floor(size(K,1)*threshold));
                        K(~(MST | knn)) = 0;
                    end

                    %% Normalization of P
                    D = eye(nRow);
                    for iRow = 1:nRow
                        D(iRow,iRow) = sum(K(iRow,:));
                    end
                    P = D^-1*K;

                    emb(iEmb).SymmP = (D^0.5)*P*(D^-0.5);

                    [emb(iEmb).EigenVectors,eVals] = eigs(emb(iEmb).SymmP,nRow);
                    emb(iEmb).EigenValues = diag(eVals);

                    if (isstring(t) || ischar(t)) 
                        if strcmp(t,'multiscale')
                            % Use multiscale method from Richards, J. W., Freeman, P. E., Lee, A. B., & Schafer, C. M. (2009). Accurate parameter estimation for star formation history in galaxies using SDSS spectra. Monthly Notices of the Royal Astronomical Society, 399(2), 1044-1057.
                            scaledEvals = emb(iEmb).EigenValues;
                            scaledEvals(2:end) = scaledEvals(2:end) ./ (1 - scaledEvals(2:end));
                            emb(iEmb).Emb = scaledEvals'.*emb(iEmb).EigenVectors;
                        else
                            error('Unknown string input for t parameter. Should be numeric or "multiscale"');
                        end
                    else
                        % Standard diffusion map
                        emb(iEmb).Emb = (emb(iEmb).EigenValues .^ emb(iEmb).t)'.*emb(iEmb).EigenVectors;
                    end
                end
            end
        end



    end
end